"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const prettyPrint_1 = require("./prettyPrint");
const logLevels_1 = require("./types/logLevels");
exports.LogLevels = logLevels_1.LogLevels;
/**
 * Provides methods for writing logs to in different log-levels. Logs are written in JSON format to stdout using console.log().
 */
class Logger {
    constructor() {
        /**
         * This governs which logs will be written. If log level is
         * INFO all logs with INFO and above will be written. So: INFO
         * WARN and ERROR.
         */
        this.level = this.setInitialLogLevel();
    }
    /**
     * Setup the meta data that will be added to all logs, this gives richer logs that will provide more value when they are sent to a log aggregator.
     */
    setup(options) {
        this.level = options.level || this.level;
        this.context = options.context || this.context;
        this.xRequestId = options.xRequestId || this.xRequestId;
        this.host = options.host || this.host;
        this.path = options.path || this.path;
        this.verb = options.verb || this.verb;
        this.query = options.query || this.query;
        this.useragent = options.useragent || this.useragent;
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    debug(logInput) {
        if (typeof logInput === 'string') {
            logInput = this.stringToLogInput(logInput);
        }
        let debugLevel = logLevels_1.LogLevels.DEBUG;
        if (debugLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.DEBUG);
            console.log(prettyPrint_1.prettyPrint(log));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    info(logInput) {
        if (typeof logInput === 'string') {
            logInput = this.stringToLogInput(logInput);
        }
        let infoLevel = logLevels_1.LogLevels.INFO;
        if (infoLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.INFO);
            console.log(prettyPrint_1.prettyPrint(log));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    warn(logInput) {
        if (typeof logInput === 'string') {
            logInput = this.stringToLogInput(logInput);
        }
        let warnLevel = logLevels_1.LogLevels.WARN;
        if (warnLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.WARN);
            console.log(prettyPrint_1.prettyPrint(log));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {
     * message: 'This is my message',
     * context: AwsLambdaContext,
     * xRequestId: 'id-from-external-system',
     * something: 'my property',
     * somethingElse: 'another extra property' }
     */
    error(logInput) {
        if (typeof logInput === 'string') {
            logInput = this.stringToLogInput(logInput);
        }
        let errorLevel = logLevels_1.LogLevels.ERROR;
        if (errorLevel >= this.level) {
            logInput = this.preProcessErrorProperties(logInput);
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.ERROR);
            console.log(prettyPrint_1.prettyPrint(log));
        }
    }
    /**
     * Set initial log level from environment param PN_LOG_LEVEL, with fallback to INFO.
     */
    setInitialLogLevel() {
        try {
            if (!process.env.PN_LOG_LEVEL) {
                return logLevels_1.LogLevels.INFO;
            }
            let level = process.env.PN_LOG_LEVEL;
            const possibleLogLevel = logLevels_1.LogLevels[level];
            // Explicit undefined check to avoid any false negatives
            if (possibleLogLevel !== undefined) {
                return possibleLogLevel;
            }
            else {
                return logLevels_1.LogLevels.INFO;
            }
        }
        catch (ex) {
            return logLevels_1.LogLevels.INFO;
        }
    }
    stringToLogInput(message) {
        return { message };
    }
    createLogJson(logInput, level) {
        // @ts-ignore
        let logOutput = {};
        let context = this.context;
        logOutput['@timestamp'] = Date.now();
        logOutput.level = logLevels_1.LogLevels[level];
        logOutput.message = logInput.message;
        if (this.xRequestId) {
            logOutput.xRequestId = this.xRequestId;
        }
        if (this.host) {
            logOutput.host = this.host;
        }
        if (this.verb) {
            logOutput.verb = this.verb;
        }
        if (this.path) {
            logOutput.path = this.path;
        }
        if (this.useragent) {
            logOutput.useragent = this.useragent;
        }
        if (this.query) {
            logOutput.query = this.query;
        }
        /**
         * By using the lambda context we can easily add more rich logging.
         */
        if (context) {
            if (context.awsRequestId) {
                logOutput.awsRequestId = context.awsRequestId;
            }
            if (context.functionName) {
                logOutput.functionName = context.functionName;
            }
            if (context.invokedFunctionArn) {
                logOutput.invokedFunctionArn = context.invokedFunctionArn;
            }
            if (context.memoryLimitInMB) {
                logOutput.memoryLimitInMB = context.memoryLimitInMB;
            }
            if (context.getRemainingTimeInMillis) {
                logOutput.remainingTime = context.getRemainingTimeInMillis();
            }
        }
        for (let prop of Object.keys(logInput)) {
            if (prop === 'message' || prop === 'xRequestId' || prop === 'context') {
                continue;
            }
            logOutput[prop] = logInput[prop];
        }
        return logOutput;
    }
    /**
     * If we find a property of type Error we cannot stringify it directly because it
     * will just be outputted as "{}", instead if we find an instanceof Error we add
     * the stack to a stack property.
     */
    preProcessErrorProperties(logInput) {
        for (let prop of Object.keys(logInput)) {
            if (logInput[prop] instanceof Error) {
                let ex = logInput[prop];
                logInput[prop] = Object.assign({}, ex, { name: ex.name, message: ex.message, stack: ex.stack.split('\n') });
            }
        }
        return logInput;
    }
}
const log = new Logger();
exports.log = log;
//# sourceMappingURL=logger.js.map